@@ -272,7 +272,7 @@ _with_guile_gtk_generic_tree_model_get_column_type (void *p)
 
     scm_obj = scm_c_gtype_instance_to_scm ((GTypeInstance*)tree_model);
     scm_ret = scm_call_2 (PROC_FROM_INSTANCE (tree_model, on_get_column_type),
-                          scm_obj, SCM_MAKINUM (a->i));
+                          scm_obj, scm_from_int (a->i));
     SCM_VALIDATE_GTYPE_CLASS_COPY (0, scm_ret, gtype);
     a->t = gtype;
     return NULL;
@@ -376,7 +376,7 @@ _with_guile_gtk_generic_tree_model_get_value (void *p)
 
     scm_obj = scm_c_gtype_instance_to_scm ((GTypeInstance*)tree_model);
     scm_ret = scm_call_3 (PROC_FROM_INSTANCE (tree_model, on_get_value),
-                          scm_obj, GET_ITER_VAL (iter), SCM_MAKINUM (a->i));
+                          scm_obj, GET_ITER_VAL (iter), scm_from_int (a->i));
 
     /* oh god this is terrible */
     _with_guile_gtk_generic_tree_model_get_column_type (a);
@@ -556,7 +556,7 @@ _with_guile_gtk_generic_tree_model_iter_nth_child (void *p)
     scm_obj = scm_c_gtype_instance_to_scm ((GTypeInstance*)tree_model);
     scm_ret = scm_call_3 (PROC_FROM_INSTANCE (tree_model, on_iter_nth_child),
                           scm_obj, GET_ITER_VAL_OR_FALSE (parent),
-                          SCM_MAKINUM (a->i));
+                          scm_from_int (a->i));
 
     if (SCM_NFALSEP (scm_ret)) {
         SET_ITER_VAL (iter, scm_ret);
